#!/bin/csh -v
#
# Build all the programs in "mini-VICAR", which contains just the parts
# needed to build certain programs used by Mars Express for data distribution.
#
# Usage:
# % build_mini_vicar.csh top-level-dir
#
# where top-level-dir is the top of the mini-VICAR tree (underneath there
# should be "vicar" and "external" directories).
#
# Note that the build is extracted from the much larger VICAR system build
# procedures.  Thus there are several things that are done that are not
# needed for mini-VICAR.  In particular, errors about not finding "taetm"
# can be ignored, as can errors about make_pdf_cache.  Compile warnings can
# also be ignored (they are uncommon; <10% of the modules depending on the
# platform, but can be ignored).
#
# If you have "word too long" problems then try "unsetenv CLASSPATH" before
# building.  Should only happen if you set up a full VICAR environment first.
#
#---------------------------------------------------------------

if ($#argv != 1) then
   echo 'Wrong parameters...'
   head -20 $0
   echo 'Wrong parameters...'
   echo
   exit
endif

setenv TO $1/vicar
setenv TOEXT $1/external

##################################################################
#
# Now build the pieces we got
#

unset verbose

echo "******** BUILDING ********"

cd $TO
setenv V2TOP $TO
source $TO/vicset1.csh
source $TO/vicset2.csh

cd $TOEXT/spice/c/src/cspice
./mkprodct.csh

\rm $TO/olb/$VICCPU/libtae.a
\rm $TO/olb/$VICCPU/libtaec.a
ln -s $TO/olb/$VICCPU/libstae.a $TO/olb/$VICCPU/libtae.a
ln -s $TO/olb/$VICCPU/libstaec.a $TO/olb/$VICCPU/libtaec.a
  
cd $TO
make -f imakefile.vicar Makefile
make -f Makefile.$VICCPU make_util make_stae make_shvic make_rtl \
	make_motifapp make_p1 make_hw make_gui make_p2


