#!/bin/csh -v
#
# Build all the Java programs in "mini-VICAR", which contains just the parts
# needed to build certain programs used by Mars Express for data distribution.
#
# Usage:
# % build_mini_vicar_java.csh top-level-dir
#
# where top-level-dir is the top of the mini-VICAR tree (underneath there
# should be "vicar" and "external" directories).
#
# Note that the build is extracted from the much larger VICAR system build
# procedures.  Thus there are several things that are done that are not
# needed for mini-VICAR.  In particular, errors about not finding "taetm"
# can be ignored, as can errors about make_pdf_cache.  Compile warnings can
# also be ignored (they are uncommon; <10% of the modules depending on the
# platform, but can be ignored).
#
# The Java build requires JDK 1.4 or later, Java Advanced Imaging (JAI)
# 1.1.2 or later, and JAI Image I/O Tools 1.0 or later.  The latter two
# packages should be properly installed inside the JDK, but it will also
# work to point to them via $CLASSPATH before invoking this script.
# 
#---------------------------------------------------------------

if ($#argv != 1) then
   echo 'Wrong parameters...'
   head -25 $0
   echo 'Wrong parameters...'
   echo
   exit
endif

setenv TO $1/vicar

##################################################################
#
# Now build the pieces we got
#

unset verbose

echo "******** BUILDING JAVA ********"

cd $TO
setenv V2TOP $TO
source $TO/vicset1.csh
source $TO/vicset2.csh

cd $TO
$V2TOP/util/java_build.csh

