package jpl.mipl.io.plugins;

import jpl.mipl.io.vicar.*;

import java.util.Locale;

import javax.imageio.ImageWriteParam;

/**
 * PDSImageWriteParam
 * @author Steve Levoe
 *
 . This class will be used to pass in some needed values useful to a PDSImageWriter
 * currently all the default items in the ImageWriteParam are IGNORED.
 * The current items are to support transcoding.
 * Someday we'll add them too.
 */
public class PDSImageWriteParam extends ImageWriteParam {

	String outputFileName; // used by the writer to put the filename into the label
	boolean transcodeIIOmetadata = false; // turns on attempt to tramscode if true
	String xslFileName = null; // transcoder xsl script
	String vicarLabelString = null ; // a String containing a complete vicar label read from the input file
	// useful for creating an emebeded vicar label
	VicarLabel vicarLabel = null; 
	boolean embedVicarLabel = false; // controls if an embeded vicar label is added to the output file
	
	boolean debug = false;
	boolean outputXML = false;
	
	boolean dirty = false; // flag to indicate if the image's data has changed
	// later we may add a way to search for a default script in the jar file
	boolean addMerItems = false; // one set of items
	boolean addStatistics = false;  // a different set of items
    boolean calculateStatistics = false; // calculate the values, otherwise placeholders are added
    
    boolean addLinePrefix = true; // if prefix metadata is present write it to the output file
    // there will be a VicarBinaryLinePrefix object in the PDSMetadata Object
    
	boolean addBinaryHeader = true; // if binary header metadata is present write it to the output file
		// there will be a VicarBinaryHeader object in the PDSMetadata Object
		
    /**
     * VicarBinaryLinePrefix vicarBinaryLinePrefix = null;
     *
     * we could also add a VicarBinaryLinePrefix Object 
     * this would allow someone to add prefix data to a PDS image where the prefix could be generated 
     * any where. OR they can add it to the PDSMetadata Object which is how it currently works
     * **/
    
    
    /* these values must be calculated somewhere and passed to the writer via params
    
    int first_line = 1;
    int first_line_sample = 1;
    String sample_bit_mask = "2#0000111111111111#furble";
    
    int invalid_constant = 0; 
    int missing_constant = 0;
    
    String mean = "UNK";
    String median = "UNK";
    String maximum = "UNK";
    String minimum = "UNK";
    String standard_deviation = "UNKNOWN";
    String checksum = "UNK";
    
    
	
	/**
	 * Constructor for PDSImageWriteParam.
	 */
	public PDSImageWriteParam() {
		super();
	}

	/**
	 * Constructor for PDSImageWriteParam.
	 * @param locale
	 */
	public PDSImageWriteParam(Locale locale) {
		super(locale);
	}
	
	public PDSImageWriteParam( String _outputFileName, // used by the writer to put the filename into the label
				boolean _transcodeIIOmetadata, // turns on attempt to transcode if true
				String _xslFileName,// transcoder xsl script
				String _vicarLabelString, // a String containing a complete vicar label read from the input file
																	// useful for creating an emebeded vicar label
				VicarLabel _vicarLabel ) {
					
		super();
		outputFileName = _outputFileName;
		transcodeIIOmetadata = _transcodeIIOmetadata; 
		xslFileName =  _xslFileName;
		vicarLabelString = _vicarLabelString;
		vicarLabel = _vicarLabel;
	}
	
	
	// getters and setters
	public void setAddMerItems(boolean f) {
    	addMerItems = f;
    }
    
      public boolean getAddMerItems() {
    	return addMerItems ;
    }
    
	public void setAddLinePrefix(boolean f) {
			addLinePrefix = f;
		}
    
	  public boolean getAddLinePrefix() {
			return addLinePrefix ;
		}
    
	public void setAddBinaryHeader(boolean f) {
				addBinaryHeader = f;
			}
    
		public boolean getAddBinaryHeader() {
				return addBinaryHeader ;
			}
			
    public void setAddStatistics(boolean f) {
    	addStatistics = f;
    }
    
      public boolean getAddStatistics() {
    	return addStatistics ;
    }
    
    public void setCalculateStatistics(boolean f) {
    	calculateStatistics = f;
    }
    
      public boolean getCalculateStatistics() {
    	return calculateStatistics ;
    }
    
    
	public String getOutputFileName() {
		return outputFileName;
	}
	
	public void setOutputFileName(String _outputFileName) {
		outputFileName = _outputFileName;
	}
	
	
	public boolean getTranscodeIIOmetadata () {
		return transcodeIIOmetadata;
	}
	public void setTranscodeIIOmetadata (boolean _transcodeIIOmetadata) {
		 transcodeIIOmetadata = _transcodeIIOmetadata;
	}
	
	public String getXslFileName () {
		return xslFileName;
	}
	public void setXslFileName (String _xslFileName) {
		 xslFileName = _xslFileName;
	}
	
	public String getVicarLabelString() {
		return vicarLabelString;
	}
	public void setVicarLabelString(String _vicarLabelString) {
		vicarLabelString =  _vicarLabelString;
	}
	
	// useful for creating an emebeded vicar label
	public VicarLabel getVicarLabel() {
		return vicarLabel;
	}
	public void setVicarLabel(VicarLabel _vicarLabel) {
		vicarLabel = _vicarLabel;
	}
	
	public void setDebug(boolean _debug) {
		debug = _debug;
	}
	public boolean getDebug() {
		return debug ;
	}

	public void setDirty(boolean d) {
		dirty = d;
	}
	
	public boolean getDirty() {
		return dirty ;
	}
	
	
	
	// the VicarLabel must also exist ?? We could still make a basici label from RenderedImage
	 public void setEmbedVicarLabel(boolean _embed) {
		embedVicarLabel = _embed;
	}
	public boolean getEmbedVicarLabel() {
		return embedVicarLabel ;
	}
	
	public void setOutputXML(boolean xml) {
		outputXML =  xml;
	}
	public boolean getOutputXML() {
		return outputXML ;
	}
	
	public String toString() {
		StringBuffer sb = new StringBuffer("PDSImageWriteParam\n");
		

	sb.append("outputFileName = "+outputFileName+"\n");
	sb.append("transcodeIIOmetadata ="+ transcodeIIOmetadata+"\n");
	sb.append("xslFileName = "+xslFileName+"\n");
	// String vicarLabelString = null ; // a String containing a complete vicar label read from the input file
	// useful for creating an emebeded vicar label
	// VicarLabel vicarLabel = null; 
	sb.append("embedVicarLabel = "+embedVicarLabel+"\n");
	
	sb.append("debug = "+debug+"\n");
	sb.append("outputXML = "+outputXML+"\n");
	
	sb.append("dirty = "+dirty+"\n");
	sb.append("addMerItems = "+addMerItems+"\n");
	sb.append("addStatistics = "+addStatistics+"\n");
    sb.append("calculateStatistics = "+calculateStatistics+"\n");
	sb.append("addLinePrefix = "+addLinePrefix+"\n");
	sb.append("addBinaryHeader = "+addBinaryHeader+"\n");

	return sb.toString();
	}
}
