#include "xvmaininc.h"
#include "defines.h"
#include "declares.h"
#include "externs.h"

/* Data-type translation functions.  Functions are supplied to translate*/
/* between all data types:  byte==unsigned char, half==short int,	*/
/* full==int, real==float, doub==double, comp==struct v2_complex.	*/
/* It is a good idea to re-write at least some of these in assembler	*/
/* code, in a host-dependent file (like "convert_vms.mar"), then use	*/
/* #if's to remove the functions here that are not needed.		*/

/************************************************************************/
/************************************************************************/

#if !VAX_ARCH

int byte2half(from, to, len, trans)
unsigned char *from;
short int *to;
int len;
struct trans *trans;
{
   register int i;

   for (i=0; i<len; i++)
      (*to++) = (short int)(*from++);
   return SUCCESS;
}

/************************************************************************/

int byte2full(from, to, len, trans)
unsigned char *from;
int *to;
int len;
struct trans *trans;
{
   register int i;

   for (i=0; i<len; i++)
      (*to++) = (int)(*from++);
   return SUCCESS;
}

/************************************************************************/

int byte2real(from, to, len, trans)
unsigned char *from;
float *to;
int len;
struct trans *trans;
{
   register int i;

   for (i=0; i<len; i++)
      (*to++) = (float)(*from++);
   return SUCCESS;
}

/************************************************************************/

int byte2doub(from, to, len, trans)
unsigned char *from;
double *to;
int len;
struct trans *trans;
{
   register int i;

   for (i=0; i<len; i++)
      (*to++) = (double)(*from++);
   return SUCCESS;
}

/************************************************************************/

int byte2comp(from, to, len, trans)
unsigned char *from;
struct v2_complex *to;
int len;
struct trans *trans;
{
   register int i;

   for (i=0; i<len; i++) {
      to->r = (float)(*from++);
      to->i = 0.0;
      to++;
   }
   return SUCCESS;
}

/************************************************************************/
/************************************************************************/

int half2byte(from, to, len, trans)
short int *from;
unsigned char *to;
int len;
struct trans *trans;
{
   register int i;

   for (i=0; i<len; i++)
      (*to++) = (unsigned char)(*from++);
   return SUCCESS;
}

/************************************************************************/

int half2full(from, to, len, trans)
short int *from;
int *to;
int len;
struct trans *trans;
{
   register int i;

   for (i=0; i<len; i++)
      (*to++) = (int)(*from++);
   return SUCCESS;
}

/************************************************************************/

int half2real(from, to, len, trans)
short int *from;
float *to;
int len;
struct trans *trans;
{
   register int i;

   for (i=0; i<len; i++)
      (*to++) = (float)(*from++);
   return SUCCESS;
}

/************************************************************************/

int half2doub(from, to, len, trans)
short int *from;
double *to;
int len;
struct trans *trans;
{
   register int i;

   for (i=0; i<len; i++)
      (*to++) = (double)(*from++);
   return SUCCESS;
}

/************************************************************************/

int half2comp(from, to, len, trans)
short int *from;
struct v2_complex *to;
int len;
struct trans *trans;
{
   register int i;

   for (i=0; i<len; i++) {
      to->r = (float)(*from++);
      to->i = 0.0;
      to++;
   }
   return SUCCESS;
}

/************************************************************************/
/************************************************************************/

int full2byte(from, to, len, trans)
int *from;
unsigned char *to;
int len;
struct trans *trans;
{
   register int i;

   for (i=0; i<len; i++)
      (*to++) = (unsigned char)(*from++);
   return SUCCESS;
}

/************************************************************************/

int full2half(from, to, len, trans)
int *from;
short int *to;
int len;
struct trans *trans;
{
   register int i;

   for (i=0; i<len; i++)
      (*to++) = (short int)(*from++);
   return SUCCESS;
}

/************************************************************************/

int full2real(from, to, len, trans)
int *from;
float *to;
int len;
struct trans *trans;
{
   register int i;

   for (i=0; i<len; i++)
      (*to++) = (float)(*from++);
   return SUCCESS;
}

/************************************************************************/

int full2doub(from, to, len, trans)
int *from;
double *to;
int len;
struct trans *trans;
{
   register int i;

   for (i=0; i<len; i++)
      (*to++) = (double)(*from++);
   return SUCCESS;
}

/************************************************************************/

int full2comp(from, to, len, trans)
int *from;
struct v2_complex *to;
int len;
struct trans *trans;
{
   register int i;

   for (i=0; i<len; i++) {
      to->r = (float)(*from++);
      to->i = 0.0;
      to++;
   }
   return SUCCESS;
}

/************************************************************************/
/************************************************************************/

int real2byte(from, to, len, trans)
float *from;
unsigned char *to;
int len;
struct trans *trans;
{
   register int i;

   for (i=0; i<len; i++)
      (*to++) = (unsigned char)(*from++);
   return SUCCESS;
}

/************************************************************************/

int real2half(from, to, len, trans)
float *from;
short int *to;
int len;
struct trans *trans;
{
   register int i;

   for (i=0; i<len; i++)
      (*to++) = (short int)(*from++);
   return SUCCESS;
}

/************************************************************************/

int real2full(from, to, len, trans)
float *from;
int *to;
int len;
struct trans *trans;
{
   register int i;

   for (i=0; i<len; i++)
      (*to++) = (int)(*from++);
   return SUCCESS;
}

/************************************************************************/

int real2doub(from, to, len, trans)
float *from;
double *to;
int len;
struct trans *trans;
{
   register int i;

   for (i=0; i<len; i++)
      (*to++) = (double)(*from++);
   return SUCCESS;
}

/************************************************************************/

int real2comp(from, to, len, trans)
float *from;
struct v2_complex *to;
int len;
struct trans *trans;
{
   register int i;

   for (i=0; i<len; i++) {
      to->r = (float)(*from++);
      to->i = 0.0;
      to++;
   }
   return SUCCESS;
}

/************************************************************************/
/************************************************************************/

int doub2byte(from, to, len, trans)
double *from;
unsigned char *to;
int len;
struct trans *trans;
{
   register int i;

   for (i=0; i<len; i++)
      (*to++) = (unsigned char)(*from++);
   return SUCCESS;
}

/************************************************************************/

int doub2half(from, to, len, trans)
double *from;
short int *to;
int len;
struct trans *trans;
{
   register int i;

   for (i=0; i<len; i++)
      (*to++) = (short int)(*from++);
   return SUCCESS;
}

/************************************************************************/

int doub2full(from, to, len, trans)
double *from;
int *to;
int len;
struct trans *trans;
{
   register int i;

   for (i=0; i<len; i++)
      (*to++) = (int)(*from++);
   return SUCCESS;
}

/************************************************************************/

int doub2real(from, to, len, trans)
double *from;
float *to;
int len;
struct trans *trans;
{
   register int i;

   for (i=0; i<len; i++)
      (*to++) = (float)(*from++);
   return SUCCESS;
}

/************************************************************************/

int doub2comp(from, to, len, trans)
double *from;
struct v2_complex *to;
int len;
struct trans *trans;
{
   register int i;

   for (i=0; i<len; i++) {
      to->r = (float)(*from++);
      to->i = 0.0;
      to++;
   }
   return SUCCESS;
}

/************************************************************************/
/************************************************************************/

int comp2byte(from, to, len, trans)
struct v2_complex *from;
unsigned char *to;
int len;
struct trans *trans;
{
   register int i;

   for (i=0; i<len; i++) {
      (*to++) = (unsigned char)from->r;
      from++;
   }
   return SUCCESS;
}

/************************************************************************/

int comp2half(from, to, len, trans)
struct v2_complex *from;
short int *to;
int len;
struct trans *trans;
{
   register int i;

   for (i=0; i<len; i++) {
      (*to++) = (short int)from->r;
      from++;
   }
   return SUCCESS;
}

/************************************************************************/

int comp2full(from, to, len, trans)
struct v2_complex *from;
int *to;
int len;
struct trans *trans;
{
   register int i;

   for (i=0; i<len; i++) {
      (*to++) = (int)from->r;
      from++;
   }
   return SUCCESS;
}

/************************************************************************/

int comp2real(from, to, len, trans)
struct v2_complex *from;
float *to;
int len;
struct trans *trans;
{
   register int i;

   for (i=0; i<len; i++) {
      (*to++) = (float)from->r;
      from++;
   }
   return SUCCESS;
}

/************************************************************************/

int comp2doub(from, to, len, trans)
struct v2_complex *from;
double *to;
int len;
struct trans *trans;
{
   register int i;

   for (i=0; i<len; i++) {
      (*to++) = (double)from->r;
      from++;
   }
   return SUCCESS;
}

/************************************************************************/
/************************************************************************/

#endif /* !VAX_ARCH */

