#include "xvmaininc.h"
#if RTL_USE_TAE
#include "taeconf.inp"
#include "parblk.inc"
#endif
#include "defines.h"
#include "declares.h"
#include "externs.h"
#include "ftnbridge.h"

/************************************************************************/
/* xvcmdout - Sends a command string to TAE to be executed, and returns	*/
/* output values in the xviparm et al parblock.				*/
/*									*/
/* The string must be an intrinsic command or a procedure PDF that uses	*/
/* only intrinsic commands, i.e. no processes, no DCL.  This is mainly	*/
/* intended for use by VIDS, but it has other applications.		*/
/*									*/
/* xvcmdout() differs from xvcommand() only in that xvcmdout() makes	*/
/* output variables accessible.  This is most useful with the JGET	*/
/* VIDS command, which returns values through the parblock mechanism	*/
/* to the caller.  The returned values are then available via xviparm	*/
/* (and the other xvip* routines)... i.e. they're in the interactive	*/
/* parblock.								*/
/*									*/
/* Historical note:  This routine started out as xvrecpar(), which	*/
/* had to be called after xvcommand().  It was implemented using	*/
/* modifications to the core of TAE.  However, with TAE 4.1 new		*/
/* features were added that allowed the same functionality built-in	*/
/* to TAE.  To take advantage of them, xvcommand() and xvrecpar() had	*/
/* to be combined into a single routine, xvcmdout().			*/
/************************************************************************/

/************************************************************************/
/* Fortran-Callable Version						*/
/************************************************************************/

void FTN_NAME(xvcmdout)(char *fcmd, int *status, ZFORSTR_PARAM)
/* char *fcmd;			In: command to be executed */
/* int *status;			Out: returned status */
{
   ZFORSTR_BLOCK
   char c_cmd[250];

   zsfor2c(c_cmd, 249, fcmd, &fcmd, 2, 1, 1, status);

   *status = zvcmdout(c_cmd);

   return;
}

/************************************************************************/
/* C-Callable Version							*/
/************************************************************************/

#if RTL_USE_TAE

int zvcmdout(cmd)
char *cmd;			/* In: command to execute */
{
   struct PARBLK parblk;
   int status;
   struct VARIABLE *sfi;
   struct VARIABLE *p_fvar();

   current_call = VCMDOUT;		/* for error handler */

/* Note that q_cmd() can abort if q_init, or q_string fail, but those	*/
/* are not likely to fail (certainly not from user input) so it should	*/
/* be okay.								*/

   status = q_cmd(cmd);
   if (status != SUCCESS) {
      error_handler(NO_UNIT, XVCOMMAND_ERROR);
      return XVCOMMAND_ERROR;
   }

   status = p_inim(iparb, P_BYTES, P_CONT);	/* Get result to iparb */
   if (status != SUCCESS) {
      error_handler(NO_UNIT, XVCOMMAND_ERROR);
      return XVCOMMAND_ERROR;
   }

   sfi = p_fvar(iparb, "$SFI");			/* Get completion code */
   if (sfi) {
      status = IVAL(*sfi, 0);
      if (status != SUCCESS) {
         error_handler(NO_UNIT, XVCOMMAND_FAIL);
         return status;
      }
      return SUCCESS;
   }

   error_handler(NO_UNIT, XVCOMMAND_ERROR);	/* $SFI not found */
   return XVCOMMAND_ERROR;

}

#else

int zvcmdout()
{
   current_call = VCMDOUT;		/* for error handler */
   error_handler(NO_UNIT, NO_TAE_SUPPORT);
   return NO_TAE_SUPPORT;
}

#endif

