#-----------------------------------------------------------------------
#
# This utility, and the companion vicset2, will set up VICAR for use.
# The top-level directory is given by the environment variable $V2TOP.
# It is prompted for if it is not set.
#
# This file must be source'd from csh, since it sets environment variables.
#
# vicset1 sets up items that are inherited by subshells or subprocesses
# (mostly environment variables).  vicset2 sets up items that are not
# inherited (mostly aliases).  This means that only vicset2 needs to be
# rerun if a subshell is created.
#
#-----------------------------------------------------------------------
#
if ($?V2TOP == 0) then
   echo -n "Enter the name of the root VICAR directory -> "
   setenv V2TOP $<
endif
# V2TOP_LC is needed instead of doing it inline due to some weird csh bug
setenv V2TOP_LC `echo $V2TOP | tr '[A-Z]' '[a-z]'`
if ("$V2TOP" != "$V2TOP_LC") then
#   echo 'Warning: TAE will not work with uppercase characters in $V2TOP.'
endif
unsetenv V2TOP_LC
#
#-----------------------------------------------------------------------
#
# Determine the CPU type for subdirectories
# This now uses "uname" to determine the platform type, if possible.
# As a backup, if "uname" isn't there, or comes up with an unknown result,
# we use the old C preprocessor trick.  The reason for using "uname" is so
# that a C compiler is not required for average user sites.
# Since the idiots at Sun did not provide a way to tell between SunOS and
# Solaris in the C preprocessor, *ALL COMPILES* and imakes and everything
# must do "-DSOLARIS" if we're on a Solaris machine.  Sigh.  Not only that,
# we must set VICAR_SOLARIS so the build procedure (imakefile.vicar) can
# bootstrap itself by doing -D$(VICAR_SOLARIS).  We can't use VICCPU because
# "sun-solr" has a dash in it and the preprocessor barfs.  Double sigh.
#
#-----------------------------------------------------------------------
#
setenv VICAR_SOLARIS NO_SOLARIS
if ($?VICCPU != 0) unsetenv VICCPU
if (-x /bin/uname) then
   if ("`uname`" == "SunOS") then
      if ("`uname -r | cut -c1`" == "5") then
         setenv VICAR_SOLARIS SOLARIS
         if ("`uname -p`" == "i386") then
            setenv VICCPU x86-solr
         else
            setenv VICCPU sun-solr
         endif
      else
# sun-3 not supported in here - modify if you need it
         setenv VICCPU sun-4
      endif
   endif
   if ("`uname`" == "IRIX") setenv VICCPU sgi
   if ("`uname`" == "IRIX64") setenv VICCPU sgi
   if ("`uname`" == "HP-UX") setenv VICCPU hp-700
   if ("`uname`" == "OSF1") setenv VICCPU axp-unix
   if ("`uname`" == "Linux") then
      if ("`uname -m`" == "i686") then
         setenv VICCPU x86-linux
      else
         setenv VICCPU axp-linux
      endif
   endif
endif

if ($?VICCPU == 0) then
   setenv VICCPU `cc -I$V2TOP/rtl/inc -E $V2TOP/util/getcpu_cpp.c | sed -e '/^$/d' -e '/#/d' | tr '[A-Z]"' '[a-z] '`
endif
#
#-----------------------------------------------------------------------
#
# Remove remnants of any old VICAR's from system environment variables
# (needed to support "select" of more than one system).
#
#-----------------------------------------------------------------------
#
# This check is for the system build.  Shouldn't happen in user calls
#
if (! -x $V2TOP/util/remove_element.csh) then
   chmod +x $V2TOP/util/remove_element.csh
endif
#
# X Resources
#
if ($?XFILESEARCHPATH != 0) then
   if ($?OLDV2TOP != 0) then
      setenv XFILESEARCHPATH `$V2TOP/util/remove_element.csh $XFILESEARCHPATH $OLDV2TOP/\*`
   endif
else
# ***NOTE*** X is really weird about XFILESEARCHPATH.  If it is not defined,
# it will look in a default place.  If it is set, this default path is not
# searched.  So, the default must be included in XFILESEARCHPATH.  But, there
# appears to be no way to query what the default is!!!  So, we just use the
# default of /usr/lib/X11/<garbage defined in XtResolvePathname()>.  If you
# have default app-defaults in a different place (e.g. /usr/X11R5/lib/X11),
# you will need to change this pathname.
   set defXdir = /usr/lib/X11
   setenv XFILESEARCHPATH $defXdir/%L/%T/%N%C%S:$defXdir/%l/%T/%N%C%S:$defXdir/%T/%N%C%S:$defXdir/%L/%T/%N%S:$defXdir/%l/%T/%N%S:$defXdir/%T/%N%S
   unset defXdir
endif
# Shared library search path
#
#
# Shared library search path.  HP uses SHLIB_PATH, everyone else uses
# LD_LIBRARY_PATH
#
if ($VICCPU == 'hp-700') then
   if ($?SHLIB_PATH != 0) then
      if ($?OLDV2TOP != 0) then
         setenv SHLIB_PATH `$V2TOP/util/remove_element.csh $SHLIB_PATH $OLDV2TOP/\*`
      endif
   else
      setenv SHLIB_PATH
   endif
else
   if ($?LD_LIBRARY_PATH != 0) then
      if ($?OLDV2TOP != 0) then
         setenv LD_LIBRARY_PATH `$V2TOP/util/remove_element.csh $LD_LIBRARY_PATH $OLDV2TOP/\*`
      endif
   else
      setenv LD_LIBRARY_PATH
   endif
endif
#
# PATH must always
# be set, so don't bother checking.  But, we'd better make sure the script
# is available, or we really hose up the path!!
#
if (-x $V2TOP/util/remove_element.csh) then
   if ($?OLDV2TOP != 0) then
      setenv PATH `$V2TOP/util/remove_element.csh $PATH $OLDV2TOP/\*`
   endif
endif
#
#-----------------------------------------------------------------------
#
# Directory Definitions
#
#
#setenv	TAE		$V2TOP/tae52
if ($VICCPU == 'axp-unix' || $VICCPU == 'x86-linux' || $VICCPU == 'axp-linux') then
#  setenv TAE		$V2TOP/tae53
endif

#setenv	V2RTL		$V2TOP/rtl
setenv	V2SOURCE	$V2TOP/rtl/source
setenv	V2INC		$V2TOP/rtl/inc

#setenv	V2OLBTOP	$V2TOP/olb
setenv	V2OLB		$V2TOP/olb/$VICCPU
#setenv	V2LIBTOP	$V2TOP/lib
setenv	V2LIB		$V2TOP/lib/$VICCPU

setenv	V2UTIL		$V2TOP/util
setenv	V2UTILEXE	$V2TOP/util/$VICCPU

#setenv	MATH77TOP	$V2TOP/math77
#setenv	MATH77SOURCE	$V2TOP/math77/source

#setenv	GUITOP		$V2TOP/gui
setenv	GUISOURCE	$V2TOP/gui/prog
setenv	GUISUB		$V2TOP/gui/sub
setenv	GUIINC		$V2TOP/gui/inc
#setenv	GUILIBTOP	$V2TOP/gui/lib
setenv	GUILIB		$V2TOP/gui/lib/$VICCPU/

setenv  SHVTOP          $V2TOP/shell_vicar
setenv  SHVSOURCE       $V2TOP/shell_vicar/prog
setenv  SHVSUB          $V2TOP/shell_vicar/sub
setenv  SHVINC          $V2TOP/shell_vicar/inc
setenv  SHVLIBDIR       $V2TOP/shell_vicar/lib/$VICCPU
setenv  SHVLIB          $V2TOP/shell_vicar/lib/$VICCPU/libshvic.a

setenv  STAETOP          $V2TOP/stae
setenv  STAESOURCE       $V2TOP/stae/source
setenv  STAESUB          $V2TOP/stae/sub
setenv  STAEINC          $V2TOP/stae/inc
setenv TAEINC $STAEINC
setenv TAELIB $V2OLB
setenv  STAECLIBDIR       $V2TOP/stae/lib/$VICCPU
setenv  STAECLIB          $V2TOP/stae/lib/$VICCPU/libstaec.a
setenv  STAELIBDIR       $V2TOP/stae/lib/$VICCPU
setenv  STAELIB          $V2TOP/stae/lib/$VICCPU/libstae.a



#setenv	P1TOP		$V2TOP/p1
#setenv	P1SOURCE	$V2TOP/p1/prog
setenv	P1SUB		$V2TOP/p1/sub
setenv	P1INC		$V2TOP/p1/inc
#setenv	P1LIBTOP	$V2TOP/p1/lib
setenv	R1LIB		$V2TOP/p1/lib/$VICCPU
setenv	P1LIB		$V2TOP/p1/lib/$VICCPU

#setenv	P2TOP		$V2TOP/p2
setenv	P2SOURCE	$V2TOP/p2/prog
setenv	P2SUB		$V2TOP/p2/sub
setenv	P2INC		$V2TOP/p2/inc
#setenv	P2LIBTOP	$V2TOP/p2/lib
setenv	R2LIB		$V2TOP/p2/lib/$VICCPU
setenv	P2LIB		$V2TOP/p2/lib/$VICCPU

#setenv	P3TOP		$V2TOP/p3
setenv	P3SOURCE	$V2TOP/p3/prog
setenv	P3SUB		$V2TOP/p3/sub
setenv	P3INC		$V2TOP/p3/inc
#setenv	P3LIBTOP	$V2TOP/p3/lib
setenv	R3LIB		$V2TOP/p3/lib/$VICCPU
setenv	P3LIB		$V2TOP/p3/lib/$VICCPU

setenv  VSLSOURCE       $V2TOP/vsl/prog
setenv  VSLSUB          $V2TOP/vsl/sub
setenv  VSLINC          $V2TOP/vsl/inc
setenv  VSLLIB          $V2TOP/vsl/lib/$VICCPU

setenv  V2DATA          $V2TOP/data

setenv  V2CONFIG        $V2TOP/config

setenv	HWSOURCE	$V2TOP/hw/prog
setenv	HWSUB		$V2TOP/hw/sub
setenv	HWINC		$V2TOP/hw/inc
#setenv	HWLIBTOP	$V2TOP/hw/lib
setenv	HWLIB		$V2TOP/hw/lib/$VICCPU

setenv  MARSTOP         $V2TOP/mars
setenv  MARSCONFIG      $MARSTOP/config
setenv  MARSDATA        $MARSTOP/data
setenv  MARSINC         $MARSTOP/inc
setenv  MARSSOURCE      $MARSTOP/src/prog
setenv  MARSRSRC        $MARSTOP/rsrc
setenv  MARSSUB         $MARSTOP/src/sub
setenv  MARSLIB         $MARSTOP/lib/$VICCPU

setenv  CASSOURCE       $V2TOP/cas/prog
setenv  CASSUB          $V2TOP/cas/sub
setenv  CASINC          $V2TOP/cas/inc
setenv  CASLIB          $V2TOP/cas/lib/$VICCPU

setenv  GLLSOURCE       $V2TOP/gll/prog
setenv  GLLSUB          $V2TOP/gll/sub
setenv  GLLINC          $V2TOP/gll/inc
setenv  GLLLIB          $V2TOP/gll/lib/$VICCPU


#setenv	SHVICTOP	$V2TOP/shell_vicar
setenv	SHVICSOURCE	$V2TOP/shell_vicar/source
setenv	SHVICINC	$V2TOP/shell_vicar/inc

setenv	VRDIFONTS	$V2TOP/vrdi/fonts
setenv	VRDIINC		$V2TOP/vrdi/inc
setenv	VRDILIB		$V2TOP/vrdi/lib/$VICCPU

setenv	VIDSLIB		$V2TOP/vids/lib/$VICCPU

setenv	MDMSINC		$V2TOP/mdms/inc
setenv	MDMSLIB		$V2TOP/mdms/lib/$VICCPU
setenv	MDMSOLB		$V2TOP/mdms/olb/$VICCPU

setenv  FEI             $V2TOP/mdms/test

setenv  V2JAVA          $V2TOP/java
setenv  V2HTML          $V2TOP/html

#-----------------------------------------------------------------------
#
# The following defines the temporary file to use for shell-VICAR output
# parameters.  It should be unique per process tree, but should NOT
# be changed in subprocesses... otherwise a subshell would create a file
# that couldn't be found by the caller (which defeats the whole purpose!).
# As long as vicset1 is run only once per process tree (i.e. check for
# inherited env vars before calling it in .cshrc), this shouldn't be a
# problem.
#
setenv V2PARAM_FILE /tmp/shvic_tmp.${USER}_$$
#
#-----------------------------------------------------------------------
#
# DEFAULTSPICE selects whether the normal mode for accessing SPICE data
# (via getspice2/putspice2) is via local SPICE kernels (LOCALSPICE) or
# remotely through the SPICE server (REMOTESPICE).  This may be overridden
# by application parameters; this merely sets up the default.
#
setenv DEFAULTSPICE LOCALSPICE
#
#
#
#-----------------------------------------------------------------------
#
# If you do not have Sybase client software, nothing is required here.
# If you do, you must merge the specified Sybase interface file in with
# your own Sybase interface file.  Contact your Sybase administrator
# or the VICAR Installation Guide for instructions.  SYBASE is only set
# if it is not already; it is not redefined.
#
if ($?SYBASE == 0) then
   setenv SYBASE	$V2CONFIG/interfaces/$VICCPU
endif

#-----------------------------------------------------------------------
# 
# The following tell FEI and the Password Server where to find their
# interface files.  They should not normally have to be changed.
# NOTE: These are for external distribution.  MIPS overrides them via selcat.
#
setenv FEI		$V2CONFIG/interfaces
setenv PWDSERVER	$V2CONFIG/interfaces
#
#-----------------------------------------------------------------------
#
# The following define locations for external software, which is optional
# software used by MIPL for some programs but not developed by MIPL.
# Change the definitions as needed for your system.
#
# Because the current setup for VICAR has most things in
# /usr/local/vicar/external or .../proprietary, we use a relative pathname:
# "$V2TOP/../external" (or proprietary).  If this is NOT the case, define
# V2EXTERNAL or V2PROPRIETARY to be the pathname of the external directory
# BEFORE running vicset1.  Of course, if you have libraries that exist
# elsewhere, you can always change individual definitions.
# THIS NEEDS TO BE CLEANED UP!!!!
#
if ($?V2EXTERNAL == 0) then
   setenv V2EXT /vicsys/external
else
   setenv V2EXT $V2EXTERNAL
endif
#
#-----------------------------------------------------------------------
# The following defines the TIFF library. The TIFF library  is a set of
# routines to read/write TIFF files.  It also includes GEOTIFF, and the
# ZLIB compression library.
setenv TIFFLIB $V2EXT/tiff/v3.4/$VICCPU
#-----------------------------------------------------------------------
#
# The following defines the library for the PVM (Parallel Virtual Machine)
# software.  There is one additional variable, PVM_ARCH, which is not set
# because it is identical to VICCPU.  If it is determined to be needed
# later, it can be added.
#
setenv	PVM_ROOT	$V2EXT/pvm/v337/pvm3
setenv	PVM_CLIB	$PVM_ROOT/lib/$VICCPU/libpvm3.a
setenv	PVM_FLIB	$PVM_ROOT/lib/$VICCPU/libfpvm3.a
setenv	PVM_INCL	$PVM_ROOT/include
setenv  PVM_BIN		$PVM_ROOT/bin/$VICCPU
#
# The following defines the PGPLOT Graphics Subroutine Library
#
setenv PGPLOTLIB        $V2EXT/pgplot/v5.2.2/$VICCPU
setenv PGPLOT_DIR       $V2EXT/pgplot/v5.2.2/$VICCPU
#-----------------------------------------------------------------------
#
# The following defines the PDS library, which is unfortunately a different
# library (it's newer) than the one below.  
# This includes the lablib3 and OAL (Object Access Library) parts of PDS.
# 
setenv PDS_LIBRARY $V2EXT/pds/v1.2/$VICCPU
# 
#
#-----------------------------------------------------------------------
#
# The following defines the library for the NAIF SPICE routines.  The
# variable should point to the top of the SPICE tree, i.e. "naif" should
# be a subdirectory under this environment variable.
#
setenv CSPICELIB $V2TOP/../external/spice/c/lib
setenv CSPICEINC $V2TOP/../external/spice/c/include
#
#
#-----------------------------------------------------------------------
#
# The following defines the location of the Kerberos library.  The machine
# dependency is due to the fact that Kerberos is included with Solaris
# (so it goes in a standard place) but is a 3rd-party product on other OS's.
#
if ($VICCPU == 'sun-solr') then
setenv	KERBEROSINC	/usr/include/kerberos
setenv	KERBEROSOLB	/usr/kerberos/lib
else
setenv	KERBEROSINC	/usr/kerberos/include
setenv	KERBEROSOLB	/usr/kerberos/lib
endif
#
#
#-----------------------------------------------------------------------
#
# The following defines the location for the Xerces C++ XML parsing
# library.  The root URL is <http://xml.apache.org/xerces-c>.  There
# you will find both the downloads and the documentation.
#
setenv XERCES_C_ROOT $V2TOP/xerces-c-src_2_6_0
#this environment is used during xerces builds
setenv XERCESCROOT $XERCES_C_ROOT
#
#-----------------------------------------------------------------------
#
# The following defines the location of the MotifApp library.  This is a
# version of the C++ class library developed by Doug Young in his book
# "Object-Oriented Programming with C++ and OSF/Motif", Prentice-Hall, 1992,
# ISBN 0-13-630252-1.  It has been extensively modified for use within
# VICAR, and will be included in VICAR (rather than being external) in the
# future.
#
setenv MOTIFAPPINC	$V2TOP/MotifApp
setenv MOTIFAPPLIB	$V2TOP/MotifApp/$VICCPU
# 
#-----------------------------------------------------------------------
#
# The following defines the location of the Math77 library.  This library
# was developed by the JPL Computational Mathematics Subgroup.  For more
# information, see <http://math.jpl.nasa.gov> or mail to
# vsnyder@math.jpl.nasa.gov.  This library has been included under the
# VICAR tree in the past; it was moved to the external category in Feb. 96.
#
setenv MATH77LIB	$V2EXT/math77/v5.0/lib/$VICCPU

#
#-----------------------------------------------------------------------
#
# The following sets up the locations for the Java includes used by native
# code.  Although they are system-wide rather than in the VICAR tree, we
# need a symbol here for portability.  Also, note that the names of the
# platform-specific subdir are NOT the same as $VICCPU... an unfortunate
# situation which makes life very messy.
#
setenv V2JAVA_SYS_INC                   /usr/java/include

if ("$VICCPU" == "sun-solr") then
  setenv V2JAVA_SYS_PLAT_INC            /usr/java/include/solaris
else
  if ("$VICCPU" == "x86-linux" || "$VICCPU" == "axp-linux" || "$VICCPU" == "ppc-linux") then
    setenv V2JAVA_SYS_PLAT_INC          /usr/java/include/linux
  else
    if ("$VICCPU" == "sgi") then
      setenv V2JAVA_SYS_PLAT_INC        /usr/java/include/irix
    else
# Punt
      setenv V2JAVA_SYS_PLAT_INC        $V2JAVA_SYS_INC
    endif
  endif
endif
#
#
#-----------------------------------------------------------------------
#
## Set up TAE
#
#
## Check for TAE version 4 or 5
if (0) then
# version 4
#  source $TAE/csh/usrassign
#  source $TAE/csh/pgmassign
else
# version 5
#  if (-f $TAE/bin/csh/taesetupclassic) then
# version 5.3 - is this really right???
#    source $TAE/bin/csh/taesetup
#    source $TAE/bin/csh/taesetupmin
#    source $TAE/bin/csh/taesetupclassic
#    source $TAE/bin/csh/taesetupsrc
  else
# version 5.2
#    source $TAE/bin/csh/taesetup
  endif
endif
#
## Set up debugging for TAE.  This is only needed for the Solaris Workshop 5.0
# compilers.  Normally you just setenv DEBUGGER to the pathname of your
## debugger, but WS5.0 requires a -D option which TAE cannot handle, thus the
# intermediate script.
#
if ($VICCPU == 'sun-solr') then
  setenv DEBUGGER $V2UTIL/workshop-D.sh
endif
#
#-----------------------------------------------------------------------
#-----------------------------------------------------------------------
#
# Paths and system environment variables
#
#-----------------------------------------------------------------------
#-----------------------------------------------------------------------
#
if (0) then
  setenv	V2FTNINC	"$V2INC $TINC  $P1INC $P2INC  $HWINC  $HWINC "
else
  setenv	V2FTNINC	"$V2INC   $P1INC $P2INC  $HWINC  $HWINC "
endif
#
# X Resources
#
# !!!! NOTE: Fix MDMS search path when they install it somewhere real **TEMP**
# Note: OS X can't handle "xvd" and "XVd".  So we name them all ".xres".
# No reason not to do that for all platforms (actually it'll make them easier
# to find).  MDMS and HW files have not been renamed; when they are, the
# ".xres" should be added to them, below.
setenv	XFILESEARCHPATH	${XFILESEARCHPATH}:${GUILIB}/%N%S.xres:${R2LIB}/%N%S.xres:${HWLIB}/%N%S:${MARSLIB}/%N%S.xres
#
# Shared library search path
#
if ($VICCPU == 'hp-700') then
   setenv SHLIB_PATH ${SHLIB_PATH}:${V2OLB}
else
# The special check for Solaris and /usr/dt/lib should be removed.  Probably
# need to put a Run-path on the link for /usr/dt/lib to fix this.
   if ($VICCPU == 'sun-solr') then
      setenv LD_LIBRARY_PATH ${LD_LIBRARY_PATH}:${V2OLB}:/usr/dt/lib
   else
      setenv LD_LIBRARY_PATH ${LD_LIBRARY_PATH}:${V2OLB}:${XERCES_C_ROOT}/lib
   endif
endif

setenv XMICONSEARCHPATH $GUILIB/%B:$R2LIB/%B
#
# Path
#
set path = ($path  $GUILIB  $V2LIB)
#
#-----------------------------------------------------------------------
#
# Finally, set OLDV2TOP so that we can remove references to this directory
# from the system environment variables later, if we need to.
#
setenv OLDV2TOP	$V2TOP
#
#-----------------------------------------------------------------------
#
#
# Classpath.  This is ugly because *each* jar file must be put into
# $CLASSPATH *individually*... you can't just specify the directory.  Argh.
# In order to avoid error messages, we do an "ls" first to see if there are
# and jar files, before doing the foreach.
# Note:  if you change this, the corresponding code in $V2UTIL/java_build.csh
# should also be changed.
#
if ($?CLASSPATH != 0) then
   if ($?OLDV2TOP != 0) then
      setenv CLASSPATH `$V2TOP/util/remove_element.csh $CLASSPATH $OLDV2TOP/\*`
   endif
else
   setenv CLASSPATH
endif

if ("$CLASSPATH" == "") then
   setenv CLASSPATH ${V2HTML}/jars
else
   setenv CLASSPATH ${CLASSPATH}:${V2HTML}/jars
endif
ls $V2HTML/jars/*.jar >& /dev/null
if ($status == 0) then
   foreach jar ($V2HTML/jars/*.jar)
      setenv CLASSPATH ${CLASSPATH}:$jar
   end
endif
#
exit

